@@ -1,28 +1,9 @@
-/*
-
-Copyright (c) 2000-2009 Chih-Chung Chang and Chih-Jen Lin
-Copyright (c) 2010,2016 Alois Schloegl <alois.schloegl@gmail.com>
-This function is part of the NaN-toolbox
-http://pub.ist.ac.at/~schloegl/matlab/NaN/
-
-This code was extracted from libsvm-mat-2.9-1 in Jan 2010 and 
-modified for the use with Octave 
-
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, see <http://www.gnu.org/licenses/>.
+/* 
+  This code was extracted from libsvm 3.2.3 in Feb 2019 and 
+  modified for the use with Octave and Matlab
 
 
-Copyright (c) 2000-2009 Chih-Chung Chang and Chih-Jen Lin
+Copyright (c) 2000-2019 Chih-Chung Chang and Chih-Jen Lin
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -52,28 +33,21 @@ PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-*/
-
+*/ 
 
 #include <stdlib.h>
 #include <string.h>
-#if defined(HAVE_EXTERNAL_LIBSVM)
-  #include <libsvm/svm.h>
-#else
-  #include "svm.h"
+#include "svm.h"
+#include "svm_model_matlab.h"
+
+
+#ifdef MX_API_VER
+#if MX_API_VER < 0x07030000
+typedef int mwIndex;
+#endif
 #endif
 
-#include "mex.h"
-
-#ifdef tmwtypes_h
-  #if (MX_API_VER<=0x07020000)
-    typedef int mwSize;
-    typedef int mwIndex;
-  #endif 
-#endif 
-
-#define NUM_OF_RETURN_FIELD 10
+#define NUM_OF_RETURN_FIELD 11
 
 #define Malloc(type,n) (type *)malloc((n)*sizeof(type))
 
@@ -83,6 +57,7 @@ static const char *field_names[] = {
 	"totalSV",
 	"rho",
 	"Label",
+	"sv_indices",
 	"ProbA",
 	"ProbB",
 	"nSV",
@@ -90,10 +65,6 @@ static const char *field_names[] = {
 	"SVs"
 };
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struct svm_model *model)
 {
 	int i, j, n;
@@ -103,7 +74,7 @@ const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struc
 
 	rhs = (mxArray **)mxMalloc(sizeof(mxArray *)*NUM_OF_RETURN_FIELD);
 
-	/* Parameters */
+	// Parameters
 	rhs[out_id] = mxCreateDoubleMatrix(5, 1, mxREAL);
 	ptr = mxGetPr(rhs[out_id]);
 	ptr[0] = model->param.svm_type;
@@ -113,19 +84,19 @@ const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struc
 	ptr[4] = model->param.coef0;
 	out_id++;
 
-	/* nr_class */
+	// nr_class
 	rhs[out_id] = mxCreateDoubleMatrix(1, 1, mxREAL);
 	ptr = mxGetPr(rhs[out_id]);
 	ptr[0] = model->nr_class;
 	out_id++;
 
-	/* total SV */
+	// total SV
 	rhs[out_id] = mxCreateDoubleMatrix(1, 1, mxREAL);
 	ptr = mxGetPr(rhs[out_id]);
 	ptr[0] = model->l;
 	out_id++;
 
-	/* rho */
+	// rho
 	n = model->nr_class*(model->nr_class-1)/2;
 	rhs[out_id] = mxCreateDoubleMatrix(n, 1, mxREAL);
 	ptr = mxGetPr(rhs[out_id]);
@@ -133,7 +104,7 @@ const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struc
 		ptr[i] = model->rho[i];
 	out_id++;
 
-	/* Label */
+	// Label
 	if(model->label)
 	{
 		rhs[out_id] = mxCreateDoubleMatrix(model->nr_class, 1, mxREAL);
@@ -145,7 +116,19 @@ const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struc
 		rhs[out_id] = mxCreateDoubleMatrix(0, 0, mxREAL);
 	out_id++;
 
-	/* probA */
+	// sv_indices
+	if(model->sv_indices)
+	{
+		rhs[out_id] = mxCreateDoubleMatrix(model->l, 1, mxREAL);
+		ptr = mxGetPr(rhs[out_id]);
+		for(i = 0; i < model->l; i++)
+			ptr[i] = model->sv_indices[i];
+	}
+	else
+		rhs[out_id] = mxCreateDoubleMatrix(0, 0, mxREAL);
+	out_id++;
+
+	// probA
 	if(model->probA != NULL)
 	{
 		rhs[out_id] = mxCreateDoubleMatrix(n, 1, mxREAL);
@@ -157,7 +140,7 @@ const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struc
 		rhs[out_id] = mxCreateDoubleMatrix(0, 0, mxREAL);
 	out_id ++;
 
-	/* probB */
+	// probB
 	if(model->probB != NULL)
 	{
 		rhs[out_id] = mxCreateDoubleMatrix(n, 1, mxREAL);
@@ -169,7 +152,7 @@ const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struc
 		rhs[out_id] = mxCreateDoubleMatrix(0, 0, mxREAL);
 	out_id++;
 
-	/* nSV */
+	// nSV
 	if(model->nSV)
 	{
 		rhs[out_id] = mxCreateDoubleMatrix(model->nr_class, 1, mxREAL);
@@ -181,7 +164,7 @@ const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struc
 		rhs[out_id] = mxCreateDoubleMatrix(0, 0, mxREAL);
 	out_id++;
 
-	/* sv_coef */
+	// sv_coef
 	rhs[out_id] = mxCreateDoubleMatrix(model->l, model->nr_class-1, mxREAL);
 	ptr = mxGetPr(rhs[out_id]);
 	for(i = 0; i < model->nr_class-1; i++)
@@ -189,11 +172,11 @@ const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struc
 			ptr[(i*(model->l))+j] = model->sv_coef[i][j];
 	out_id++;
 
-	/* SVs */
+	// SVs
 	{
 		int ir_index, nonzero_element;
 		mwIndex *ir, *jc;
-		mxArray *pprhs[1], *pplhs[1];	
+		mxArray *pprhs[1], *pplhs[1];
 
 		if(model->param.kernel_type == PRECOMPUTED)
 		{
@@ -205,7 +188,7 @@ const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struc
 			nonzero_element = 0;
 			for(i = 0; i < model->l; i++) {
 				j = 0;
-				while(model->SV[i][j].index != -1) 
+				while(model->SV[i][j].index != -1)
 				{
 					nonzero_element++;
 					j++;
@@ -213,18 +196,18 @@ const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struc
 			}
 		}
 
-		/* SV in column, easier accessing */
+		// SV in column, easier accessing
 		rhs[out_id] = mxCreateSparse(num_of_feature, model->l, nonzero_element, mxREAL);
 		ir = mxGetIr(rhs[out_id]);
 		jc = mxGetJc(rhs[out_id]);
 		ptr = mxGetPr(rhs[out_id]);
-		jc[0] = ir_index = 0;		
+		jc[0] = ir_index = 0;
 		for(i = 0;i < model->l; i++)
 		{
 			if(model->param.kernel_type == PRECOMPUTED)
 			{
-				/* make a (1 x model->l) matrix */
-				ir[ir_index] = 0; 
+				// make a (1 x model->l) matrix
+				ir[ir_index] = 0;
 				ptr[ir_index] = model->SV[i][0].value;
 				ir_index++;
 				jc[i+1] = jc[i] + 1;
@@ -234,14 +217,14 @@ const char *model_to_matlab_structure(mxArray *plhs[], int num_of_feature, struc
 				int x_index = 0;
 				while (model->SV[i][x_index].index != -1)
 				{
-					ir[ir_index] = model->SV[i][x_index].index - 1; 
+					ir[ir_index] = model->SV[i][x_index].index - 1;
 					ptr[ir_index] = model->SV[i][x_index].value;
 					ir_index++, x_index++;
 				}
 				jc[i+1] = jc[i] + x_index;
 			}
 		}
-		/* transpose back to SV in row */
+		// transpose back to SV in row
 		pprhs[0] = rhs[out_id];
 		if(mexCallMATLAB(1, pplhs, 1, pprhs, "transpose"))
 			return "cannot transpose SV matrix";
@@ -272,7 +255,7 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 	mxArray **rhs;
 
 	num_of_fields = mxGetNumberOfFields(matlab_struct);
-	if(num_of_fields != NUM_OF_RETURN_FIELD) 
+	if(num_of_fields != NUM_OF_RETURN_FIELD)
 	{
 		*msg = "number of return field is not correct";
 		return NULL;
@@ -287,8 +270,9 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 	model->probA = NULL;
 	model->probB = NULL;
 	model->label = NULL;
+	model->sv_indices = NULL;
 	model->nSV = NULL;
-	model->free_sv = 1; /* XXX */
+	model->free_sv = 1; // XXX
 
 	ptr = mxGetPr(rhs[id]);
 	model->param.svm_type = (int)ptr[0];
@@ -306,7 +290,7 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 	model->l = (int)ptr[0];
 	id++;
 
-	/* rho */
+	// rho
 	n = model->nr_class * (model->nr_class-1)/2;
 	model->rho = (double*) malloc(n*sizeof(double));
 	ptr = mxGetPr(rhs[id]);
@@ -314,7 +298,7 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 		model->rho[i] = ptr[i];
 	id++;
 
-	/* label */
+	// label
 	if(mxIsEmpty(rhs[id]) == 0)
 	{
 		model->label = (int*) malloc(model->nr_class*sizeof(int));
@@ -324,7 +308,17 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 	}
 	id++;
 
-	/* probA */
+	// sv_indices
+	if(mxIsEmpty(rhs[id]) == 0)
+	{
+		model->sv_indices = (int*) malloc(model->l*sizeof(int));
+		ptr = mxGetPr(rhs[id]);
+		for(i=0;i<model->l;i++)
+			model->sv_indices[i] = (int)ptr[i];
+	}
+	id++;
+
+	// probA
 	if(mxIsEmpty(rhs[id]) == 0)
 	{
 		model->probA = (double*) malloc(n*sizeof(double));
@@ -334,7 +328,7 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 	}
 	id++;
 
-	/* probB */
+	// probB
 	if(mxIsEmpty(rhs[id]) == 0)
 	{
 		model->probB = (double*) malloc(n*sizeof(double));
@@ -344,7 +338,7 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 	}
 	id++;
 
-	/* nSV */
+	// nSV
 	if(mxIsEmpty(rhs[id]) == 0)
 	{
 		model->nSV = (int*) malloc(model->nr_class*sizeof(int));
@@ -354,7 +348,7 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 	}
 	id++;
 
-	/* sv_coef */
+	// sv_coef
 	ptr = mxGetPr(rhs[id]);
 	model->sv_coef = (double**) malloc((model->nr_class-1)*sizeof(double));
 	for( i=0 ; i< model->nr_class -1 ; i++ )
@@ -364,16 +358,16 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 			model->sv_coef[i][j] = ptr[i*(model->l)+j];
 	id++;
 
-	/* SV */
+	// SV
 	{
-		int sr, sc, elements;
+		int sr, elements;
 		int num_samples;
 		mwIndex *ir, *jc;
 		mxArray *pprhs[1], *pplhs[1];
 
-		/* transpose SV */
+		// transpose SV
 		pprhs[0] = rhs[id];
-		if(mexCallMATLAB(1, pplhs, 1, pprhs, "transpose")) 
+		if(mexCallMATLAB(1, pplhs, 1, pprhs, "transpose"))
 		{
 			svm_free_and_destroy_model(&model);
 			*msg = "cannot transpose SV matrix";
@@ -382,7 +376,6 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 		rhs[id] = pplhs[0];
 
 		sr = (int)mxGetN(rhs[id]);
-		sc = (int)mxGetM(rhs[id]);
 
 		ptr = mxGetPr(rhs[id]);
 		ir = mxGetIr(rhs[id]);
@@ -395,7 +388,7 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 		model->SV = (struct svm_node **) malloc(sr * sizeof(struct svm_node *));
 		x_space = (struct svm_node *)malloc(elements * sizeof(struct svm_node));
 
-		/* SV is in column */
+		// SV is in column
 		for(i=0;i<sr;i++)
 		{
 			int low = (int)jc[i], high = (int)jc[i+1];
@@ -403,7 +396,7 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 			model->SV[i] = &x_space[low+i];
 			for(j=low;j<high;j++)
 			{
-				model->SV[i][x_index].index = (int)ir[j] + 1; 
+				model->SV[i][x_index].index = (int)ir[j] + 1;
 				model->SV[i][x_index].value = ptr[j];
 				x_index++;
 			}
@@ -416,8 +409,3 @@ struct svm_model *matlab_matrix_to_model(const mxArray *matlab_struct, const cha
 
 	return model;
 }
-
-#ifdef __cplusplus
-}
-#endif
-
